﻿/* Program.cs
 */

#region Using directives

using System;
using System.Configuration.Install;
using System.Reflection;
using System.ServiceProcess;

#endregion

namespace Always64
{
    class ServiceInstallerUtility
    {
        private static readonly string exePath 
            = Assembly.GetExecutingAssembly().Location;

        // ====================================================================

        public static bool Install()
        {
            try
            {
                ManagedInstallerClass.InstallHelper
                    (
                        new[] {exePath}
                    );
            }
            catch
            {
                return false;
            }
            return true;
        }

        // ====================================================================

        public static bool Uninstall()
        {
            try
            {
                ManagedInstallerClass.InstallHelper
                    (
                        new[] {"/u", exePath}
                    );
            }
            catch
            {
                return false;
            }
            return true;
        }
    }

    // ====================================================================

    static class Program
    {
        static void RunService ()
        {
            ServiceBase[] ServicesToRun = new ServiceBase[] 
                                              { 
                                                  new AlwaysService() 
                                              };
            ServiceBase.Run(ServicesToRun);
        }

        // ====================================================================

        static void InstallService ( )
        {
            ServiceInstallerUtility.Install ();
        }

        // ====================================================================

        static void UninstallService ( )
        {
            ServiceInstallerUtility.Uninstall ();
        }

        // ====================================================================

        static void StartService ( )
        {
            try
            {
                ServiceController controller
                    = new ServiceController ( "AlwaysService" );
                controller.Start ();
            }
            catch ( Exception ex )
            {
                Console.WriteLine ( ex );
            }
        }

        // ====================================================================

        static void StopService ( )
        {
            try
            {
                ServiceController controller
                    = new ServiceController( "AlwaysService");
                controller.Stop ();
            }
            catch ( Exception ex )
            {
                Console.WriteLine ( ex );
            }
        }

        // ====================================================================

        static void RunAsConsoleApplication ( )
        {
            Console.WriteLine ( "IRBIS-PROXY version {0}", IrbisProxy.Version );
            Console.WriteLine ( "Running as console application" );
            Console.WriteLine ();
            IrbisProxy.Initialize ();
            IrbisProxy.ListenerRoutine ();
        }

        // ====================================================================

        static void ShowVersion()
        {
            Console.WriteLine
                (
                    "Always64 - IBRIS-PROXY service\r\n"
                    + "version {0}",
                    IrbisProxy.Version
                );
        }

        // ====================================================================

        static void ShowHelp ()
        {
            Console.WriteLine 
                (
                    "Always64 - IRBIS-PROXY service\r\n"
                  + "\r\n"
                  + "-install \tinstall the service\r\n"
                  + "-uninstall \tuninstall the service\r\n"
                  + "-start \t\tstart the service\r\n"
                  + "-stop \t\tstop the service\r\n"
                  + "-console \trun the service as console application\r\n"
                  + "-version \tshow version and exit\r\n"
                  + "-help \tshow this screen and exit\r\n"
                  + "\r\n"
                  + "no arguments runs the service as console application"
                );
        }

        // ====================================================================

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main
            (
                string[] args
            )
        {
            if ( args.Length == 0 )
            {
                if ( Environment.UserInteractive )
                {
                    RunAsConsoleApplication ();
                }
                else
                {
                    RunService ();
                }
            }
            else if ( args.Length == 1 )
            {
                switch ( args [ 0 ].ToLowerInvariant() )
                {
                    case "-install":
                    case "/install":
                    case "-i":
                    case "/i":
                        InstallService ();
                        break;

                    case "-uninstall":
                    case "/uninstall":
                    case "-u":
                    case "/u":
                        UninstallService ();
                        break;

                    case "-start":
                    case "/start":
                    case "-run":
                    case "/run":
                    case "-r":
                    case "/r":
                    case "/1":
                    case "-1":
                        StartService ();
                        break;

                    case "-stop":
                    case "/stop":
                    case "-s":
                    case "/s":
                    case "-0":
                    case "/0":
                        StopService ();
                        break;

                    case "-console":
                    case "/console":
                    case "-c":
                    case "/c":
                        RunAsConsoleApplication ();
                        break;

                    case "-version":
                    case "/version":
                    case "-v":
                    case "/v":
                        ShowVersion();
                        break;

                    default:
                        ShowHelp ();
                        break;
                }
            }
            else
            {
                ShowHelp ();
            }
        }
    }
}
